//Biblioteca de calculo dos ganhos PID (Kp, Ki, Kd)
//Compativel com ESP32, Arduino e placas compatíveis  
//método Ziegler-Nichols
//Desenvolvida por Opller

#ifndef OP_AUTOTUNE_H
#define OP_AUTOTUNE_H

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

//Inclusão das bibliotecas necessárias ao controle
#include <Arduino.h>

class Autotune {
  public:
    // Constants for Ziegler-Nichols tuning mode
    typedef enum {
      ZNModeBasicPID,
      ZNModeLessOvershoot,
      ZNModeNoOvershoot
    } ZNMode;

    Autotune();

    // Configura Auto Tune
    // target: setpoint
    // interval: intervalo de atualização em microsegundos
    // type: tipo de saida (0 = digital e 1 = analogica)
    // min: valor minimo do range analogico
    // max: valor maximo do range analogico 
    // znMode: modo Ziegler-Nichols (znModeBasicPID, znModeLessOvershoot, znModeNoOvershoot)
    // tuneCycles: numero de ciclos (default 10)

    void config(int target, unsigned long interval, int min, int max, int tuneCycles, int potencia);
    void setZNMode(ZNMode zn);

    // Precisa ser chamada imediatamente antes do loop
    void start(unsigned long us);

    // Executa o Auto Tune
    // enable: habilita a saida (0 = desabilitado e 1 = habilitado)
    float execute(bool enable, int input, unsigned long us);

    // Ler os valores dos ganhos calculados
    float getKp();
    float getKi();
    float getKd();

    bool isFinished(); //Retorna 1 se o ciclo terminou

    int getCycle(); // Retorna o ciclo 

  private:
    int targetInputValue = 0;
    unsigned long loopInterval = 0;
    int minOutput, maxOutput;
    ZNMode znMode = ZNModeNoOvershoot;
    int cycles = 10;
    int percentual_potencia;
    int i;
    bool output;
    int outputValue;
    long millisseconds, t1, t2, tHigh, tLow;
    int max, min;
    float pAverage, iAverage, dAverage;
    bool enable_control;
    float kp, ki, kd;
};

#endif
