//Biblioteca de calculo dos ganhos PID (Kp, Ki, Kd)
//Compativel com ESP32, Arduino e placas compatíveis  
//método Ziegler-Nichols
//Desenvolvida por Opller

//Exemplo de utilização em controle de temperatura 
//Sensor PT100
//CLP OPX001 (Opller)
 

#include <OP_AutoTune.h>

//Inclusão da biblioteca base
#include <OPX001_IO.h>

//Inclusão biblioteca SPI (caso necessário)
#include <SPI.h>

//Definicao do numero de IOs (DI, AI, AT, DO, AO, E2P)
OPX001_IO OPX(18, 8, 4, 18, 4, 1);

//Pinos auxiliares
#define ENTX_pin 02 //Pino de comunicação RS485
#define SDA2 32
#define SCL2 33
#define MOSI 13
#define MISO 12
#define SCLK 14
#define CS 15

float targetInputValue = 2300;
unsigned long loopInterval = 100;
long millisseconds;
int output;
int input;
uint32_t TMR_DIGITAL_OUTPUT = 0;
int out_digital = 0;
uint32_t tPWM = 10000;
uint32_t tON = 0;

Autotune tune = Autotune();

void setup() {
 
  Serial.begin(115200); // (USB)
  //Inicializa o OPX001
  OPX.setup();
    
    //(setpoint, intervalo de atualização, min saida, max saida, numero de ciclos, potencia)
    //O valor do setpoint tem que estar no mesmo intervalo da saida (min-max) 
    tune.config(targetInputValue,loopInterval,0,4095,2,100);

    // Configura o modo Ziegler-Nichols 
    // Autotune::ZNModeBasicPID, Autotune::ZNModeLessOvershoot,
    // or Autotune::ZNModeNoOvershoot. Default é ZNModeNoOvershoot 
    tune.setZNMode(Autotune::ZNModeNoOvershoot);

    // Precisa ser chamado imediatamente antes do loop
    // Precisa ser chamado com o tempo atual em microsegundos
    tune.start(millis());

    TMR_DIGITAL_OUTPUT = millis();
}

void loop() {

     // Faz o ciclo até tune.isFinished = 1, ou seja, completar o numero de ciclos escolhido
    if (!tune.isFinished()){
     if (millis() - millisseconds > loopInterval) {
        
        // mesmo tempo que o controle iniciou
        millisseconds = millis();

        // variável de entrada que precisa ser controlada 
        input = OPX.AT(1);
        
        //1 = habilitado, 0 = desabilitado 
        output = tune.execute(1,input,millisseconds);

        Serial.print("input = ");
        Serial.println(input);
        Serial.print("output = ");
        Serial.println(output);
        Serial.println(tune.getCycle()); //numero do ciclo
     }
     tON = (output*tPWM)/(4095 - 0);
     if (millis() < TMR_DIGITAL_OUTPUT + tON){
     out_digital = 1;
     } else if ((millis() >= TMR_DIGITAL_OUTPUT + tON) && (millis() < TMR_DIGITAL_OUTPUT + tPWM)){
     out_digital = 0; 
     } else if (millis() >= TMR_DIGITAL_OUTPUT + tPWM){
     TMR_DIGITAL_OUTPUT = millis();
     }
     OPX.DO(16,out_digital);
   } else {
     OPX.DO(16,0);
    //Ganhos PID 
    double kp = tune.getKp();
    double ki = tune.getKi();
    double kd = tune.getKd();

    Serial.print("kp = ");
    Serial.println(kp);
    Serial.print("ki = ");
    Serial.println(ki);
    Serial.print("kd = ");
    Serial.println(kd);
   }
   OPX.poll();
}
